unit Childwin;

interface

uses Windows, Classes, Graphics, Forms, Controls, StdCtrls, ComCtrls,
  ExtCtrls, OleCtrls, SHDocVw, Buttons, SysUtils, Menus;

type
  TMDIChild = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    StatusBar: TStatusBar;
    BrowserBackButton: TSpeedButton;
    BrowserForwardButton: TSpeedButton;
    BrowserStopRefreshButton: TSpeedButton;
    BrowserHomeButton: TSpeedButton;
    CloseButton: TButton;
    URLEdit: TEdit;
    Timer: TTimer;
    WebBrowser: TWebBrowser;
    CustomButton: TButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CloseButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure URLEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure WebBrowserNewWindow2(Sender: TObject; var ppDisp: IDispatch;
      var Cancel: WordBool);
    procedure TimerTimer(Sender: TObject);
    procedure WebBrowserBeforeNavigate2(Sender: TObject;
      const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure WebBrowserCommandStateChange(Sender: TObject;
      Command: Integer; Enable: WordBool);
    procedure BrowserForwardButtonClick(Sender: TObject);
    procedure BrowserBackButtonClick(Sender: TObject);
    procedure WebBrowserDocumentComplete(Sender: TObject;
      const pDisp: IDispatch; var URL: OleVariant);
    procedure WebBrowserDownloadBegin(Sender: TObject);
    procedure WebBrowserStatusTextChange(Sender: TObject;
      const Text: WideString);
    procedure WebBrowserTitleChange(Sender: TObject;
      const Text: WideString);
    procedure WebBrowserProgressChange(Sender: TObject; Progress,
      ProgressMax: Integer);
    procedure BrowserStopRefreshButtonClick(Sender: TObject);
    procedure WebBrowserDownloadComplete(Sender: TObject);
    procedure CustomButtonClick(Sender: TObject);
    procedure BrowserHomeButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

implementation

uses Main, DebugUnit;
{$R *.DFM}

VAR CloseMe:Boolean;

procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  IF DebugForm.Visible THEN Debug('Window "'+WebBrowser.LocationURL+'" closed');
  IF MainForm.NOEXIT
    AND (MainForm.MDIChildCount = 1)
    AND (MainForm.AutoURL.Count > 0) THEN PostMessage(MainForm.Handle,IPCMessage,1,0);
end;

procedure TMDIChild.CloseButtonClick(Sender: TObject);
begin
Close;
end;

procedure TMDIChild.FormCreate(Sender: TObject);
begin
  CloseMe:=FALSE;
// Disable URL Edit 
  IF MainForm.NOURL THEN URLEdit.Enabled:=FALSE;
  IF DebugForm.Visible THEN Debug('New Window: ' + WebBrowser.LocationURL);
  WITH MainForm.CustomButton DO
    IF URL <> '' THEN
    BEGIN
      CustomButton.Caption:=Caption;
      CustomButton.Hint:=Hint;
      CustomButton.Visible:=TRUE;
    END;

end;

procedure TMDIChild.URLEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  IF Key=13 THEN
  try
    WebBrowser.Navigate(URLEdit.Text);
  except
  on E:Exception do Close;
  END;
end;

procedure TMDIChild.WebBrowserNewWindow2(Sender: TObject;
  var ppDisp: IDispatch; var Cancel: WordBool);
begin
   WITH TMDIChild.Create(Application) DO
     ppDisp:=WebBrowser.ControlInterface;
   IF DebugForm.Visible THEN Debug('New Window');
end;

procedure TMDIChild.TimerTimer(Sender: TObject);
begin
  IF CloseMe OR (NOT WebBrowser.HandleAllocated) THEN
  BEGIN
    IF DebugForm.Visible THEN Debug('WebBrowser Disappeared or asked me to close');
    Close;
  END;

end;

procedure TMDIChild.WebBrowserBeforeNavigate2(Sender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
VAR URL_:WideString;
    dummyb:Byte;
begin
  //Application.MessageBox('CheckURL','Info',MB_OK);
  Cancel:=TRUE;
  URL_:=UpperCase(URL);
  dummyb:=0;
  IF MainForm.IncFilter<>nil THEN
    WHILE (dummyb<MainForm.IncFilter.Count) AND Cancel DO
    BEGIN
      Cancel:=NOT (Pos(MainForm.IncFilter[dummyb],URL_)>0);
      Inc(dummyb);
    END;
  dummyb:=0;
  IF NOT Cancel AND (MainForm.ExcFilter<>nil) THEN
    WHILE (dummyb<MainForm.ExcFilter.Count) AND NOT Cancel DO
    BEGIN
      Cancel:=(Pos(MainForm.ExcFilter[dummyb],URL_)>0);
      Inc(dummyb);
    END;
  IF MainForm.IncFilter=nil THEN Cancel:=FALSE;
  IF (URL='about:NavigationCanceled') OR (Pos('RES:',URL_)>0) THEN Cancel:=TRUE
  ELSE IF Cancel THEN
  BEGIN
    Beep;
    Application.MessageBox(pchar(FilterErrorText),pchar(string(URL)),MB_ICONSTOP);
  END
  ELSE
  // site is OK, let's browse there
  BEGIN
    // about:version check
    IF (Pos('VERSION',URL_)>0) AND (pos('VERSION',URL_)<12) THEN
    BEGIN
      WebBrowser.Navigate(AboutString);
      Cancel:=TRUE;
    END;
    Flags:=14;
    // 14 is max restriction: no cache, no history etc. look at msdn.microsoft.com for info
  END;
  IF DebugForm.Visible THEN
    IF Cancel THEN Debug('Navigate to "'+URL+'" cancelled')
    ELSE Debug('Navigate to "'+URL+'" allowed');
end;

procedure TMDIChild.WebBrowserCommandStateChange(Sender: TObject;
  Command: Integer; Enable: WordBool);
begin
{
This enables/disables the back/forward buttons. See WebBrowser doc for info
}
begin
  CASE Command OF
    1: BrowserForwardButton.Enabled:=Enable;
    2: BrowserBackButton.Enabled:=Enable;
  END;
end;

end;

procedure TMDIChild.BrowserForwardButtonClick(Sender: TObject);
begin
   try
    WebBrowser.GoForward;
  except
    on E:Exception do ;
  end;

end;

procedure TMDIChild.BrowserBackButtonClick(Sender: TObject);
begin
   try
    WebBrowser.GoBack;
  except
    on E:Exception do ;
  end;

end;



procedure TMDIChild.WebBrowserDocumentComplete(Sender: TObject;
  const pDisp: IDispatch; var URL: OleVariant);
begin
// if this is the final DocComplete call then the pDisp is the webbrowser
// if it is not, then it means that a frame or other object completed the
// download.
  IF NOT (pDisp = WebBrowser.ControlInterface) THEN EXIT;
  URLEdit.Text:=URL;
  StatusBar.Panels[0].Text:='';
  Caption:=WebBrowser.LocationName;
  WebBrowserDownloadComplete(Sender);
  IF DebugForm.Visible THEN Debug('Completed loading ' + URL);
end;

procedure TMDIChild.WebBrowserDownloadBegin(Sender: TObject);
begin
  WITH BrowserStopRefreshbutton DO
  BEGIN
    Glyph:=MainForm.Icons[0];
    Tag:=0;
    Hint:='Stop';
  END;
end;

procedure TMDIChild.WebBrowserStatusTextChange(Sender: TObject;
  const Text: WideString);
begin
  StatusBar.Panels[1].Text:=Text;
end;

procedure TMDIChild.WebBrowserTitleChange(Sender: TObject;
  const Text: WideString);
begin
  Caption:=Text;
end;

procedure TMDIChild.WebBrowserProgressChange(Sender: TObject; Progress,
  ProgressMax: Integer);
begin
  IF (Progress>-1) AND (ProgressMax>0) THEN StatusBar.Panels[0].Text:=IntToStr(Round(Progress*100/ProgressMax))+ '%'
  ELSE IF Progress=0 THEN StatusBar.Panels[0].Text:='';

end;

procedure TMDIChild.BrowserStopRefreshButtonClick(Sender: TObject);
begin
  IF BrowserStopRefreshButton.Tag=1 THEN
    WebBrowser.Refresh
  ELSE
    WebBrowser.Stop;
end;

procedure TMDIChild.WebBrowserDownloadComplete(Sender: TObject);
begin
  WITH BrowserStopRefreshbutton DO
  BEGIN
    Glyph:=MainForm.Icons[1];
    Tag:=1;
    Hint:='Refresh';
  END;
end;

procedure TMDIChild.CustomButtonClick(Sender: TObject);
begin
  WebBrowser.Navigate(MainForm.CustomButton.URL);
end;

procedure TMDIChild.BrowserHomeButtonClick(Sender: TObject);
begin
  WebBrowser.Navigate(HomeURL);
end;


end.
